"""
Bibliothèque version 3 adaptée à Test_V3.py
"""
import pygame

c_fond = (131, 166, 151) # couleurs ( R, V B )
c_rouge = (96, 80, 220)
c_ROUGE = (102, 0, 255) # pion activé pour être déplacé
c_rouge_D = (165, 38, 10)
c_ROUGE_D = (222, 41, 22)
c_bleu = (44, 117, 255)
c_BLEU = (115, 194, 251) # pion activé pour être déplacé
c_bleu_D = (0, 204, 203)
c_BLEU_D = (0, 255, 255)

# ########################################################## pion
class pion ( ):
    """
    Couleur : Rouge -> True / Bleu => False
    position : (ligne, colonne)
    plat => plateau de jeu liste de 8x8
    """
    index_actif = -1 # -1 -> aucun pion n'est actif
    # ======================================================= __init__
    def __init__ ( self, couleur:bool, position:tuple, screen:object ):
        """
        """
        self.__couleur = couleur
        self.__lgn = position[0]
        self.__col = position[1]
        self.__dame = False
        self.__actif = False
        self.placer(screen) # Placer le pion sur le plateau
    # ======================================================= placer
    def placer ( self, screen:object ):
        x = (self.__col-1)*50+25 # position du centre
        y = (self.__lgn-1)*50+25
        pygame.draw.circle(screen, self.__couleur, (x,y), 20)
        if self.__dame :
            name_font = pygame.font.Font('led_ttf.ttf', 25)
            name = name_font.render("D", True, (255,255,150))
            screen.blit(name, (x-10,y-20))
    # ======================================================= effacer
    def effacer ( self, screen:object ):
        x = (self.__col-1)*50+25 # position du centre
        y = (self.__lgn-1)*50+25
        pygame.draw.circle(screen, c_fond, (x,y), 20)
    # ======================================================= placer
    def desactiver ( self, screen ):
        if self.__couleur == c_ROUGE :
            self.__couleur = c_rouge
        if self.__couleur == c_ROUGE_D :
            self.__couleur = c_rouge_D
        if self.__couleur == c_BLEU :
            self.__couleur = c_bleu
        if self.__couleur == c_BLEU_D :
            self.__couleur = c_bleu_D
        self.__actif = False # activer le pion
        self.placer( screen ) # modifier l'aspect
    # ======================================================= placer
    def activer ( self, screen, liste_pions ):
        """
        Modifie la couleur du pion pour le montrer "actif"
        """
        for pion in liste_pions: # s'il y a un pion actif, le désactiver
            if pion.__actif:
                pion.desactiver( screen ) 
        if self.__couleur == c_rouge :
            self.__couleur = c_ROUGE
        if self.__couleur == c_rouge_D :
            self.__couleur = c_ROUGE_D
        if self.__couleur == c_bleu :
            self.__couleur = c_BLEU
        if self.__couleur == c_bleu_D :
            self.__couleur = c_BLEU_D
        self.__actif = True # activer le pion
        self.placer( screen ) # modifier l'aspect
    # ======================================================= deplacer
    def deplacer ( self, screen:object, case:tuple ) :
        self.effacer( screen )
        self.__lgn, self.__col = case
        if self.__lgn==8 and 'R' in self.couleur and not self.__dame :
            self.__couleur = c_rouge_D
            self.__dame = True
        if self.__lgn==1 and 'B' in self.couleur and not self.__dame :
            self.__couleur = c_bleu_D
            self.__dame = True
        self.desactiver( screen )
    # ======================================================= deplacer
    def sauter ( self, screen:object, pion:object, case:tuple, pions:list ) :
        pion_a_supprimer = pion.disparaitre ( screen )
        del pion
        self.deplacer ( screen, case )
    # ======================================================= disparaître
    def disparaitre ( self, screen:object ) :
        global pions
        self.effacer(screen)
        for n in range(len(pions)) :
            if pions[n].position == self.position :
                pions.pop(n) # !!! suppression du pion de la liste
                break;
    @property
    def dame ( self )->bool :
        return self.__dame
            
    # ======================================================= actif
    @property
    def actif ( self )->bool:
        return self.__actif
    # ======================================================= actif
    @property
    def couleur ( self )->str:
        if self.__couleur==c_rouge or self.__couleur == c_ROUGE :
            return 'R'
        if self.__couleur==c_rouge_D or self.__couleur == c_ROUGE_D :
            return 'RD'
        if self.__couleur==c_bleu or self.__couleur == c_BLEU :
            return 'B'
        if self.__couleur==c_bleu_D or self.__couleur == c_BLEU_D :
            return 'BD'
    # ======================================================= position
    @property
    def position ( self )->tuple :
        return (self.__lgn, self.__col)
# =========================================== debut_jeu
def debut_jeu ( screen ):
    n=0
    screen.fill(c_fond)
    pygame.display.set_caption("Jeu de dames ...")
    clock = pygame.time.Clock()
    name_font = pygame.font.Font('led_ttf.ttf', 35)
    name = name_font.render("Jeu de Dames", True, (255,255,0))
    while n<180:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                sys.exit()      
        screen.fill(c_fond)      
        screen.blit(name, (40,140))
        pygame.display.flip()
        clock.tick(60)
        n += 1
# ================================================================ dessiner_plateau      
def dessiner_plateau ( screen:object ):
    carre_noir = lambda x, y: pygame.draw.rect(screen, (50,10,0),
                                    pygame.Rect(x,y,50,50))
    screen.fill(c_fond) # couleur de fond du plateau
    for lgn in range(4): 
        corner_y = lgn*100 # 4 doubles lignes 0, 100, 200, 300
        for col in range(8): # distribuer 8 carrés décalés alternativement
            corner_x = col*50
            carre_noir ( corner_x, corner_y+50*(col%2))
    pygame.display.flip() 
# ==================================================== tester_fin_jeu
def tester_fin_jeu ( screen:object, pions:list ) -> bool :
    rouges = 0
    bleus = 0
    for p in pions: # compter les pions
        if p.couleur=="R" or p.couleur=="RD" :
            rouges +=1
        if p.couleur=="B" or p.couleur=="BD" :
            bleus +=1
    if not rouges or not bleus:
        gagnant = "ROUGES" if not bleus else "BLEUS"
        fin_jeu ( screen, gagnant ) # afficher le gagnant
        return True
    else:
        return False
# ============================================ fin_jeu
def fin_jeu (screen, texte:str ):
    n=0
    screen.fill(c_fond)
    clock = pygame.time.Clock()
    bravo_font = pygame.font.Font('led_ttf.ttf', 40)
    bravo = bravo_font.render("## BRAVO ##", True, (255,255,0))
    gagnant_font = pygame.font.Font('led_ttf.ttf', 30)
    gagnant1 = gagnant_font.render("Les "+texte, True, (255,255,0))
    gagnant2 = gagnant_font.render("ont gagné !", True, (255,255,0))
    while n<240:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                sys.exit()      
        screen.fill(c_fond)      
        screen.blit(bravo, (40,50))
        screen.blit(gagnant1, (40,200))
        screen.blit(gagnant2, (40,270))
        pygame.display.flip()
        clock.tick(60)
        n += 1

pygame.init()
screen = pygame.display.set_mode( (400,400) )
pions=[] # Liste des pions rouges et bleus
